/*!
 * @imqueue/type-graphql-dependency - Declarative GraphQL dependency loading
 *
 * Copyright (c) 2019, imqueue.com <support@imqueue.com>
 *
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
 * REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
 * INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
 * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
 * OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 */
import { DataInitializer, DataLoader, GraphQLDependency } from '@imqueue/graphql-dependency';
import { GraphQLSchema } from 'graphql';
export declare type CreateSchemaHook = (schema: GraphQLSchema) => void;
export declare const schemaHooks: CreateSchemaHook[];
export declare function onCreateSchema(handler: CreateSchemaHook): void;
export interface DependentTypeRelations {
    as: string;
    filter: {
        [foreignField: string]: string;
    };
}
export declare type DependentType = Function | Function[];
export interface DependsOptions<T> {
    require?: [() => DependentType, DependentTypeRelations[]][];
    init?: DataInitializer<T>;
    load?: DataLoader<T>;
}
export interface DependencyInterface<T> {
    (type: Function): GraphQLDependency<T>;
    schema?: GraphQLSchema;
}
/**
 * Abstracts Dependency from @imqueue/graphql-dependency to make it possible
 * to work with a plain classes from type-graphql. This function is not intended
 * to be used to define dependencies, only to refer already defined
 * dependencies.
 *
 * To define dependencies for type-graphql classes use @DependencyFor()
 * decorator instead.
 *
 * @example
 * import { Dependency } from './src/decorators';
 * // ... in a resolver, before data return:
 * await Dependency(Consumer).load(context, data, fields);
 *
 * @param {Function} type
 * @return {GraphQLDependency<any>}
 * @constructor
 */
export declare const Dependency: DependencyInterface<any>;
/**
 * Decorator for describing dependencies between graphql entities, using
 * their type-graphql definitions as classes and @imqueue/graphql-dependency
 * as an engine.
 *
 * This decorator simply wrap native graphql-js-based implementation of
 * graphql-dependency from @imqueue and operates with a similar concepts:
 *  - requires
 *  - initializers
 *  - loaders
 * @see https://github.com/imqueue/graphql-dependency
 *
 * @example
 * // Let's assume we want to decorate Consumer objects to be dependent on
 * // nested ApiKey objects:
 * @DependencyFor<Partial<Consumer>>({
 *    // this defines dependency relations for Consumer object.
 *    // refers to: @imqueue/graphql-dependency:Dependency.require()
 *    require: [
 *        [() => ApiKey, [
 *          { as: 'apiKeys', filter: { 'consumerId': 'id' } }],
 *        ],
 *    ],
 *    // this defines initializer for Consumer, all dependencies will wait
 *    // for initializer to finish before load
 *    // refers to: @imqueue/graphql-dependency:Dependency.defineInitializer()
 *    async init(
 *        context: Context,
 *        result: Partial<Consumer>,
 *        fields?: FieldsInput,
 *    ): Promise<DataInitializerResult> {
 *        // ... do initializer stuff here ...
 *        return result;
 *    },
 *    // this defines loader for Consumer entity, which should be used by
 *    // other entities, which depend on Consumer
 *    // refers to: @imqueue/graphql-dependency:Dependency.defineLoader()
 *    async load(
 *        context: Context,
 *        filter: ConsumerListInput,
 *        fields?: FieldsInput,
 *    ): Promise<Partial<Consumer>[]> {
 *        const { data } = await context.consumer.listConsumer(filter, fields);
 *        return toConsumers(data);
 *    },
 * })
 * @ObjectType()
 * export class Consumer {
 *     // ... Consumer fields definitions goes here ...
 * }
 *
 * // now within a resolver:
 * import { Dependency } from './src/decorators';
 * // ... in a resolver, before data return:
 * await Dependency(Consumer).load(data, context, fields);
 *
 * @param {DependsOptions<T>} options
 * @return {(target: any) => void}
 * @constructor
 */
export declare function DependencyFor<T>(options: DependsOptions<T>): (target: any) => void;
